@@ -2,9 +2,8 @@
 mod test;
 mod wg;
 
-cfg_not_async!(
-    use crossbeam_channel::{tick, RecvError};
-);
+cfg_async!(use crate::sleep;);
+cfg_not_async!(use crossbeam_channel::{tick, RecvError};);
 
 use crate::{
     bounded,
@@ -12,7 +11,7 @@ use crate::{
     ttl::ExpirationMap,
     cache::wg::WaitGroup,
     metrics::MetricType,
-    select, sleep, spawn, stop_channel,
+    select, spawn, stop_channel,
     store::{ShardedMap, UpdateResult},
     ttl::Time,
     unbounded,
@@ -34,7 +33,7 @@ const DEFAULT_INSERT_BUF_SIZE: usize = 32 * 1024;
 // const DEFAULT_BUFFER_ITEMS: usize = 64;
 const DEFAULT_CLEANUP_DURATION: Duration = Duration::from_millis(500);
 
-pub(crate) enum Item<V> {
+enum Item<V> {
     New {
         key: u64,
         conflict: u64,
@@ -89,6 +88,101 @@ impl<V> Item<V> {
     }
 }
 
+/// The `CacheBuilder` struct is used when creating Cache instances if you want to customize the Cache settings.
+/// 
+/// - **num_counters**
+///
+///     `num_counters` is the number of 4-bit access counters to keep for admission and eviction.
+///     Dgraph's developers have seen good performance in setting this to 10x the number of items
+///     you expect to keep in the cache when full.
+/// 
+///     For example, if you expect each item to have a cost of 1 and `max_cost` is 100, set `num_counters` to 1,000.
+///     Or, if you use variable cost values but expect the cache to hold around 10,000 items when full,
+///     set num_counters to 100,000. The important thing is the *number of unique items* in the full cache,
+///     not necessarily the `max_cost` value.
+/// 
+/// - **max_cost**
+/// 
+///     `max_cost` is how eviction decisions are made. For example, if max_cost is 100 and a new item
+///     with a cost of 1 increases total cache cost to 101, 1 item will be evicted.
+/// 
+///     `max_cost` can also be used to denote the max size in bytes. For example,
+///     if max_cost is 1,000,000 (1MB) and the cache is full with 1,000 1KB items,
+///     a new item (that's accepted) would cause 5 1KB items to be evicted.
+/// 
+///     `max_cost` could be anything as long as it matches how you're using the cost values when calling [`insert`].
+///
+/// - **key_builder**
+///
+///     [`KeyBuilder`] is the hashing algorithm used for every key. In Stretto, the Cache will never store the real key.
+///     The key will be processed by [`KeyBuilder`]. Stretto has two default built-in key builder,
+///     one is [`TransparentKeyBuilder`], the other is [`DefaultKeyBuilder`]. If your key implements [`TransparentKey`] trait,
+///     you can use [`TransparentKeyBuilder`] which is faster than [`DefaultKeyBuilder`]. Otherwise, you should use [`DefaultKeyBuilder`]
+///     You can also write your own key builder for the Cache, by implementing [`KeyBuilder`] trait.
+///
+///     Note that if you want 128bit hashes you should use the full `(u64, u64)`,
+///     otherwise just fill the `u64` at the `0` position, and it will behave like
+///     any 64bit hash.
+///
+/// - **buffer_size**
+/// 
+///     `buffer_size` is the size of the insert buffers. The Dgraph's developers find that 32 * 1024 gives a good performance.
+/// 
+///     If for some reason you see insert performance decreasing with lots of contention (you shouldn't),
+///     try increasing this value in increments of 32 * 1024. This is a fine-tuning mechanism
+///     and you probably won't have to touch this.
+/// 
+/// - **metrics**
+/// 
+///     Metrics is true when you want real-time logging of a variety of stats.
+///     The reason this is a [`CacheBuilder`] flag is because there's a 10% throughput performance overhead.
+/// 
+/// - **ignore_internal_cost**
+///
+///     Set to true indicates to the cache that the cost of
+///     internally storing the value should be ignored. This is useful when the
+///     cost passed to set is not using bytes as units. Keep in mind that setting
+///     this to true will increase the memory usage.
+///
+/// - **cleanup_duration**
+///
+///     The Cache will cleanup the expired values every 500ms by default.
+///
+/// - **update_validator**
+/// 
+///     By default, the Cache will always update the value if the value already exists in the cache.
+///     [`UpdateValidator`] is a trait to support customized update policy (check if the value should be updated
+///     if the value already exists in the cache).
+///
+/// - **callback**
+/// 
+///     [`CacheCallback`] is for customize some extra operations on values when related event happens..
+///
+/// - **coster**
+/// 
+///     [`Coster`] is a trait you can pass to the [`CacheBuilder`] in order to evaluate
+///     item cost at runtime, and only for the [`insert`] calls that aren't dropped (this is
+///     useful if calculating item cost is particularly expensive, and you don't want to
+///     waste time on items that will be dropped anyways).
+/// 
+///     To signal to Stretto that you'd like to use this Coster trait:
+/// 
+///     1. Set the Coster field to your own Coster implementation.
+///     2. When calling [`insert`] for new items or item updates, use a cost of 0.
+///
+/// - **hasher**
+///
+///     The hasher for the Cache, default is SipHasher.
+///
+/// [`Cache`]: struct.Cache.html
+/// [`TransparentKey`]: struct.TransparentKey.html
+/// [`TransparentKeyBuilder`]: struct.TransparentKeyBuilder.html
+/// [`DefaultKeyBuilder`]: struct.DefaultKeyBuilder.html
+/// [`KeyBuilder`]: trait.KeyBuilder.html
+/// [`insert`]: struct.Cache.html#method.insert
+/// [`UpdateValidator`]: trait.UpdateValidator.html
+/// [`CacheCallback`]: trait.CacheCallback.html
+/// [`Coster`]: trait.Coster.html
 pub struct CacheBuilder<
     K: Hash + Eq,
     V: Send + Sync + 'static,
@@ -96,9 +190,7 @@ pub struct CacheBuilder<
     C: Coster<V>,
     U: UpdateValidator<V>,
     CB: CacheCallback<V>,
-    PS: BuildHasher + Clone + 'static,
-    ES: BuildHasher + Clone + 'static,
-    SS: BuildHasher + Clone + 'static,
+    S: BuildHasher + Clone + 'static,
 > {
     /// metrics determines whether cache statistics are kept during the cache's
     /// lifetime. There *is* some overhead to keeping statistics, so you should
@@ -146,11 +238,7 @@ pub struct CacheBuilder<
 
     callback: Option<CB>,
 
-    policy_hasher: Option<PS>,
-
-    expiration_hasher: Option<ES>,
-
-    store_hasher: Option<SS>,
+    hasher: Option<S>,
 
     marker_k: PhantomData<fn(K)>,
     marker_v: PhantomData<fn(V)>,
@@ -159,6 +247,24 @@ pub struct CacheBuilder<
 /// Cache is a thread-safe implementation of a hashmap with a TinyLFU admission
 /// policy and a Sampled LFU eviction policy. You can use the same Cache instance
 /// from as many threads as you want.
+///
+/// 
+/// # Features
+/// * **Internal Mutability** - Do not need to use `Arc<RwLock<Cache<...>>` for concurrent code, you just need `Arc<Cache<...>`
+/// * **Sync and Async** - Stretto support async by `tokio` and sync by `crossbeam`.
+///   * In sync, Cache starts two extra OS level threads. One is policy thread, the other is writing thread.
+///   * In async, Cache starts two extra green threads. One is policy thread, the other is writing thread.
+/// * **Store policy** Stretto only store the value, which means the cache does not store the key. 
+/// * **High Hit Ratios** - with our unique admission/eviction policy pairing, Ristretto's performance is best in class.
+///     * **Eviction: SampledLFU** - on par with exact LRU and better performance on Search and Database traces.
+///     * **Admission: TinyLFU** - extra performance with little memory overhead (12 bits per counter).
+/// * **Fast Throughput** - we use a variety of techniques for managing contention and the result is excellent throughput.
+/// * **Cost-Based Eviction** - any large new item deemed valuable can evict multiple smaller items (cost could be anything).
+/// * **Fully Concurrent** - you can use as many threads as you want with little throughput degradation.
+/// * **Metrics** - optional performance metrics for throughput, hit ratios, and other stats.
+/// * **Simple API** - just figure out your ideal [`CacheBuilder`] values and you're off and running.
+/// 
+/// [`CacheBuilder`]: struct.CacheBuilder.html
 pub struct Cache<
     K,
     V,
@@ -166,83 +272,69 @@ pub struct Cache<
     C = DefaultCoster<V>,
     U = DefaultUpdateValidator<V>,
     CB = DefaultCacheCallback<V>,
-    PS = RandomState,
-    ES = RandomState,
-    SS = RandomState,
+    S = RandomState,
 > where
     K: Hash + Eq,
     V: Send + Sync + 'static,
     KH: KeyBuilder<K>,
 {
     /// store is the central concurrent hashmap where key-value items are stored.
-    pub(crate) store: Arc<ShardedMap<V, U, SS, ES>>,
+    store: Arc<ShardedMap<V, U, S, S>>,
 
     /// policy determines what gets let in to the cache and what gets kicked out.
-    pub(crate) policy: Arc<LFUPolicy<PS>>,
+    policy: Arc<LFUPolicy<S>>,
 
-    /// set_buf is a buffer allowing us to batch/drop Sets during times of high
+    /// insert_buf is a buffer allowing us to batch/drop Sets during times of high
     /// contention.
-    pub(crate) insert_buf_tx: Sender<Item<V>>,
+    insert_buf_tx: Sender<Item<V>>,
 
-    pub(crate) stop_tx: Sender<()>,
+    stop_tx: Sender<()>,
 
-    pub(crate) clear_tx: UnboundedSender<()>,
+    clear_tx: UnboundedSender<()>,
 
-    pub(crate) callback: Arc<CB>,
+    callback: Arc<CB>,
 
-    pub(crate) key_to_hash: KH,
+    key_to_hash: KH,
 
-    pub(crate) is_closed: AtomicBool,
+    is_closed: AtomicBool,
 
-    pub(crate) coster: Arc<C>,
+    coster: Arc<C>,
 
-    pub(crate) ignore_internal_cost: bool,
-
-    pub(crate) cleanup_duration: Duration,
-
-    pub(crate) metrics: Arc<Metrics>,
-
-    pub(crate) item_size: usize,
+    /// the metrics for the cache
+    pub metrics: Arc<Metrics>,
 
     _marker: PhantomData<fn(K)>,
 }
 
-struct CacheProcessor<V, C, U, CB, PS, ES, SS>
+struct CacheProcessor<V, U, CB, S>
     where
         V: Send + Sync + 'static,
-        C: Coster<V>,
         U: UpdateValidator<V>,
         CB: CacheCallback<V>,
-        PS: BuildHasher + Clone + 'static,
-        ES: BuildHasher + Clone + 'static,
-        SS: BuildHasher + Clone + 'static,
+        S: BuildHasher + Clone + 'static,
 {
-    pub(crate) insert_buf_rx: Receiver<Item<V>>,
-    pub(crate) stop_rx: Receiver<()>,
-    pub(crate) clear_rx: UnboundedReceiver<()>,
-    pub(crate) metrics: Arc<Metrics>,
-    pub(crate) store: Arc<ShardedMap<V, U, SS, ES>>,
-    pub(crate) policy: Arc<LFUPolicy<PS>>,
-    pub(crate) start_ts: HashMap<u64, Time>,
-    pub(crate) num_to_keep: usize,
-    pub(crate) callback: Arc<CB>,
-    pub(crate) coster: Arc<C>,
-    pub(crate) ignore_internal_cost: bool,
-    pub(crate) item_size: usize,
+    insert_buf_rx: Receiver<Item<V>>,
+    stop_rx: Receiver<()>,
+    clear_rx: UnboundedReceiver<()>,
+    metrics: Arc<Metrics>,
+    store: Arc<ShardedMap<V, U, S, S>>,
+    policy: Arc<LFUPolicy<S>>,
+    start_ts: HashMap<u64, Time>,
+    num_to_keep: usize,
+    callback: Arc<CB>,
+    ignore_internal_cost: bool,
+    item_size: usize,
     cleanup_duration: Duration,
 }
 
-struct CacheCleaner<'a, V, C, U, CB, PS, ES, SS>
+struct CacheCleaner<'a, V, U, CB, S>
     where
         V: Send + Sync + 'static,
-        C: Coster<V>,
         U: UpdateValidator<V>,
         CB: CacheCallback<V>,
-        PS: BuildHasher + Clone + 'static,
-        ES: BuildHasher + Clone + 'static,
-        SS: BuildHasher + Clone + 'static,
+        S: BuildHasher + Clone + 'static,
 {
-    processor: &'a mut CacheProcessor<V, C, U, CB, PS, ES, SS>,
+    processor: &'a mut CacheProcessor<V, U, CB, S>,
 }
 
 impl<K: Hash + Eq, V: Send + Sync + 'static, KH: KeyBuilder<K>>
@@ -254,10 +346,9 @@ CacheBuilder<
     DefaultUpdateValidator<V>,
     DefaultCacheCallback<V>,
     RandomState,
-    RandomState,
-    RandomState,
 >
 {
+    /// Create a new CacheBuilder
     #[inline]
     pub fn new(num_counters: usize, max_cost: i64, index: KH) -> Self {
         Self {
@@ -267,8 +358,6 @@ CacheBuilder<
             insert_buffer_size: DEFAULT_INSERT_BUF_SIZE,
             metrics: false,
             callback: Some(DefaultCacheCallback::default()),
-            policy_hasher: Some(RandomState::default()),
-            expiration_hasher: Some(RandomState::default()),
             key_to_hash: index,
             update_validator: Some(DefaultUpdateValidator::default()),
             coster: Some(DefaultCoster::default()),
@@ -276,12 +365,12 @@ CacheBuilder<
             cleanup_duration: DEFAULT_CLEANUP_DURATION,
             marker_k: Default::default(),
             marker_v: Default::default(),
-            store_hasher: Some(RandomState::default()),
+            hasher: Some(RandomState::default()),
         }
     }
 }
 
-impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS>
+impl<K, V, KH, C, U, CB, S> CacheBuilder<K, V, KH, C, U, CB, S>
     where
         K: Hash + Eq,
         V: Send + Sync + 'static,
@@ -289,10 +378,18 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
         C: Coster<V>,
         U: UpdateValidator<V>,
         CB: CacheCallback<V>,
-        PS: BuildHasher + Clone + 'static,
-        ES: BuildHasher + Clone + 'static,
-        SS: BuildHasher + Clone + 'static,
+        S: BuildHasher + Clone + 'static,
 {
+    /// Set the number of counters for the Cache.
+    ///
+    /// `num_counters` is the number of 4-bit access counters to keep for admission and eviction.
+    /// Dgraph's developers have seen good performance in setting this to 10x the number of items
+    /// you expect to keep in the cache when full.
+    ///
+    /// For example, if you expect each item to have a cost of 1 and `max_cost` is 100, set `num_counters` to 1,000.
+    /// Or, if you use variable cost values but expect the cache to hold around 10,000 items when full,
+    /// set num_counters to 100,000. The important thing is the *number of unique items* in the full cache,
+    /// not necessarily the `max_cost` value.
     #[inline]
     pub fn set_num_counters(self, num_counters: usize) -> Self {
         Self {
@@ -301,8 +398,6 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             insert_buffer_size: self.insert_buffer_size,
             metrics: self.metrics,
             callback: self.callback,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
             key_to_hash: self.key_to_hash,
             update_validator: self.update_validator,
             coster: self.coster,
@@ -310,10 +405,20 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             cleanup_duration: self.cleanup_duration,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
         }
     }
 
+    /// Set the max_cost for the Cache.
+    ///
+    /// `max_cost` is how eviction decisions are made. For example, if max_cost is 100 and a new item
+    /// with a cost of 1 increases total cache cost to 101, 1 item will be evicted.
+    ///
+    /// `max_cost` can also be used to denote the max size in bytes. For example,
+    /// if max_cost is 1,000,000 (1MB) and the cache is full with 1,000 1KB items,
+    /// a new item (that's accepted) would cause 5 1KB items to be evicted.
+    ///
+    /// `max_cost` could be anything as long as it matches how you're using the cost values when calling `insert`.
     #[inline]
     pub fn set_max_cost(self, max_cost: i64) -> Self {
         Self {
@@ -327,14 +432,19 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set the insert buffer size for the Cache.
+    ///
+    /// `buffer_size` is the size of the insert buffers. The Dgraph's developers find that 32 * 1024 gives a good performance.
+    ///
+    /// If for some reason you see insert performance decreasing with lots of contention (you shouldn't),
+    /// try increasing this value in increments of 32 * 1024.
+    /// This is a fine-tuning mechanism and you probably won't have to touch this.
     #[inline]
     pub fn set_buffer_size(self, sz: usize) -> Self {
         Self {
@@ -348,14 +458,16 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set whether record the metrics or not.
+    ///
+    /// Metrics is true when you want real-time logging of a variety of stats.
+    /// The reason this is a CacheBuilder flag is because there's a 10% throughput performance overhead.
     #[inline]
     pub fn set_metrics(self, val: bool) -> Self {
         Self {
@@ -369,14 +481,17 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set whether ignore the internal cost or not.
+    ///
+    /// By default, when [`insert`] a value in the Cache, there will always 56 for internal cost,
+    /// because the size of stored item in Cache is 56(excluding the size of value).
+    /// Set it to true to ignore the internal cost.
     #[inline]
     pub fn set_ignore_internal_cost(self, val: bool) -> Self {
         Self {
@@ -390,14 +505,13 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: val,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set the cleanup ticker for Cache, each tick the Cache will clean the expired entries.
     #[inline]
     pub fn set_cleanup_duration(self, d: Duration) -> Self {
         Self {
@@ -411,19 +525,33 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: d,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set the [`KeyBuilder`] for the Cache
+    ///
+    /// [`KeyBuilder`] is the hashing algorithm used for every key. In Stretto, the Cache will never store the real key.
+    /// The key will be processed by [`KeyBuilder`]. Stretto has two default built-in key builder,
+    /// one is [`TransparentKeyBuilder`], the other is [`DefaultKeyBuilder`]. If your key implements [`TransparentKey`] trait,
+    /// you can use [`TransparentKeyBuilder`] which is faster than [`DefaultKeyBuilder`]. Otherwise, you should use [`DefaultKeyBuilder`]
+    /// You can also write your own key builder for the Cache, by implementing [`KeyBuilder`] trait.
+    ///
+    /// Note that if you want 128bit hashes you should use the full `(u64, u64)`,
+    /// otherwise just fill the `u64` at the `0` position, and it will behave like
+    /// any 64bit hash.
+    ///
+    /// [`KeyBuilder`]: trait.KeyBuilder.html
+    /// [`TransparentKey`]: trait.TransparentKey.html
+    /// [`TransparentKeyBuilder`]: struct.TransparentKeyBuilder.html
+    /// [`DefaultKeyBuilder`]: struct.DefaultKeyBuilder.html
     #[inline]
     pub fn set_key_builder<NKH: KeyBuilder<K>>(
         self,
         index: NKH,
-    ) -> CacheBuilder<K, V, NKH, C, U, CB, PS, ES, SS> {
+    ) -> CacheBuilder<K, V, NKH, C, U, CB, S> {
         CacheBuilder {
             num_counters: self.num_counters,
             max_cost: self.max_cost,
@@ -435,19 +563,28 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set the coster for the Cache.
+    ///
+    /// [`Coster`] is a trait you can pass to the [`CacheBuilder`] in order to evaluate
+    /// item cost at runtime, and only for the [`insert`] calls that aren't dropped (this is
+    /// useful if calculating item cost is particularly expensive, and you don't want to
+    /// waste time on items that will be dropped anyways).
+    ///
+    /// To signal to Stretto that you'd like to use this [`Coster`] trait:
+    ///
+    /// 1. Set the [`Coster`] field to your own [`Coster`] implementation.
+    /// 2. When calling [`insert`] for new items or item updates, use a cost of 0.
     #[inline]
     pub fn set_coster<NC: Coster<V>>(
         self,
         coster: NC,
-    ) -> CacheBuilder<K, V, KH, NC, U, CB, PS, ES, SS> {
+    ) -> CacheBuilder<K, V, KH, NC, U, CB, S> {
         CacheBuilder {
             num_counters: self.num_counters,
             max_cost: self.max_cost,
@@ -459,19 +596,22 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: Some(coster),
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set the update validator for the Cache.
+    ///
+    /// By default, the Cache will always update the value if the value already exists in the cache.
+    /// [`UpdateValidator`] is a trait to support customized update policy (check if the value should be updated
+    /// if the value already exists in the cache).
     #[inline]
     pub fn set_update_validator<NU: UpdateValidator<V>>(
         self,
         uv: NU,
-    ) -> CacheBuilder<K, V, KH, C, NU, CB, PS, ES, SS> {
+    ) -> CacheBuilder<K, V, KH, C, NU, CB, S> {
         CacheBuilder {
             num_counters: self.num_counters,
             max_cost: self.max_cost,
@@ -483,19 +623,20 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set the callbacks for the Cache.
+    ///
+    /// [`CacheCallback`] is for customize some extra operations on values when related event happens.
     #[inline]
     pub fn set_callback<NCB: CacheCallback<V>>(
         self,
         cb: NCB,
-    ) -> CacheBuilder<K, V, KH, C, U, NCB, PS, ES, SS> {
+    ) -> CacheBuilder<K, V, KH, C, U, NCB, S> {
         CacheBuilder {
             num_counters: self.num_counters,
             max_cost: self.max_cost,
@@ -507,19 +648,19 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: self.hasher,
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Set the hasher for the Cache.
+    /// Default is SipHasher.
     #[inline]
-    pub fn set_policy_hasher<NPS: BuildHasher + Clone + 'static>(
+    pub fn set_hasher<NS: BuildHasher + Clone + 'static>(
         self,
-        hasher: NPS,
-    ) -> CacheBuilder<K, V, KH, C, U, CB, NPS, ES, SS> {
+        hasher: NS,
+    ) -> CacheBuilder<K, V, KH, C, U, CB, NS> {
         CacheBuilder {
             num_counters: self.num_counters,
             max_cost: self.max_cost,
@@ -531,64 +672,15 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             coster: self.coster,
             ignore_internal_cost: self.ignore_internal_cost,
             cleanup_duration: self.cleanup_duration,
-            policy_hasher: Some(hasher),
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: self.store_hasher,
+            hasher: Some(hasher),
             marker_k: self.marker_k,
             marker_v: self.marker_v,
         }
     }
 
+    /// Build Cache and start all threads needed by the Cache.
     #[inline]
-    pub fn set_expiration_hasher<NES: BuildHasher + Clone + 'static>(
-        self,
-        hasher: NES,
-    ) -> CacheBuilder<K, V, KH, C, U, CB, PS, NES, SS> {
-        CacheBuilder {
-            num_counters: self.num_counters,
-            max_cost: self.max_cost,
-            insert_buffer_size: self.insert_buffer_size,
-            metrics: self.metrics,
-            callback: self.callback,
-            key_to_hash: self.key_to_hash,
-            update_validator: self.update_validator,
-            coster: self.coster,
-            ignore_internal_cost: self.ignore_internal_cost,
-            cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: Some(hasher),
-            store_hasher: self.store_hasher,
-            marker_k: self.marker_k,
-            marker_v: self.marker_v,
-        }
-    }
-
-    #[inline]
-    pub fn set_store_hasher<NSS: BuildHasher + Clone + 'static>(
-        self,
-        hasher: NSS,
-    ) -> CacheBuilder<K, V, KH, C, U, CB, PS, ES, NSS> {
-        CacheBuilder {
-            num_counters: self.num_counters,
-            max_cost: self.max_cost,
-            insert_buffer_size: self.insert_buffer_size,
-            metrics: self.metrics,
-            callback: self.callback,
-            key_to_hash: self.key_to_hash,
-            update_validator: self.update_validator,
-            coster: self.coster,
-            ignore_internal_cost: self.ignore_internal_cost,
-            cleanup_duration: self.cleanup_duration,
-            policy_hasher: self.policy_hasher,
-            expiration_hasher: self.expiration_hasher,
-            store_hasher: Some(hasher),
-            marker_k: self.marker_k,
-            marker_v: self.marker_v,
-        }
-    }
-
-    #[inline]
-    pub fn finalize(self) -> Result<Cache<K, V, KH, C, U, CB, PS, ES, SS>, CacheError> {
+    pub fn finalize(self) -> Result<Cache<K, V, KH, C, U, CB, S>, CacheError> {
         let num_counters = self.num_counters;
 
         if num_counters == 0 {
@@ -609,18 +701,18 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
         let (stop_tx, stop_rx) = stop_channel();
         let (clear_tx, clear_rx) = unbounded();
 
-        let expiration_map = ExpirationMap::with_hasher(self.expiration_hasher.unwrap());
+
+        let hasher = self.hasher.unwrap();
+        let expiration_map = ExpirationMap::with_hasher(hasher.clone());
 
         let store = Arc::new(ShardedMap::with_validator_and_hasher(
             expiration_map,
             self.update_validator.unwrap(),
-            self.store_hasher.unwrap(),
+            hasher.clone(),
         ));
 
         let mut policy =
-            LFUPolicy::with_hasher(num_counters, max_cost, self.policy_hasher.unwrap())?;
-
-        let item_size = store.item_size();
+            LFUPolicy::with_hasher(num_counters, max_cost, hasher.clone())?;
 
         let coster = Arc::new(self.coster.unwrap());
         let callback = Arc::new(self.callback.unwrap());
@@ -643,7 +735,6 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             stop_rx,
             clear_rx,
             metrics.clone(),
-            coster.clone(),
             callback.clone(),
         ).spawn();
 
@@ -657,10 +748,7 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
             clear_tx,
             is_closed: AtomicBool::new(false),
             coster,
-            ignore_internal_cost: self.ignore_internal_cost,
-            cleanup_duration: self.cleanup_duration,
             metrics,
-            item_size,
             _marker: Default::default(),
         };
 
@@ -669,11 +757,15 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> CacheBuilder<K, V, KH, C, U, CB, PS, ES, SS
 }
 
 impl<K: Hash + Eq, V: Send + Sync + 'static, KH: KeyBuilder<K>> Cache<K, V, KH> {
+    /// Returns a Cache instance with default configruations.
     #[inline]
     pub fn new(num_counters: usize, max_cost: i64, index: KH) -> Result<Self, CacheError> {
         CacheBuilder::new(num_counters, max_cost, index).finalize()
     }
 
+    /// Returns a [`CacheBuilder`].
+    ///
+    /// [`CacheBuilder`]: struct.CacheBuilder.html
     #[inline]
     pub fn builder(
         num_counters: usize,
@@ -687,14 +779,12 @@ impl<K: Hash + Eq, V: Send + Sync + 'static, KH: KeyBuilder<K>> Cache<K, V, KH>
         DefaultUpdateValidator<V>,
         DefaultCacheCallback<V>,
         RandomState,
-        RandomState,
-        RandomState,
     > {
         CacheBuilder::new(num_counters, max_cost, index)
     }
 }
 
-impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
+impl<K, V, KH, C, U, CB, S> Cache<K, V, KH, C, U, CB, S>
     where
         K: Hash + Eq,
         V: Send + Sync + 'static,
@@ -702,14 +792,11 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
         C: Coster<V>,
         U: UpdateValidator<V>,
         CB: CacheCallback<V>,
-        PS: BuildHasher + Clone + 'static,
-        ES: BuildHasher + Clone + 'static,
-        SS: BuildHasher + Clone + 'static,
+        S: BuildHasher + Clone + 'static,
 {
-    /// `get` returns the value (if any) and a boolean representing whether the
-    /// value was found or not. The value can be nil and the boolean can be true at
-    /// the same time.
-    pub fn get(&self, key: &K) -> Option<ValueRef<V, SS>> {
+    /// `get` returns a `Option<ValueRef<V, SS>>` (if any) representing whether the
+    /// value was found or not.
+    pub fn get(&self, key: &K) -> Option<ValueRef<V, S>> {
         if self.is_closed.load(Ordering::SeqCst) {
             return None;
         }
@@ -728,10 +815,9 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
         }
     }
 
-    /// `get_mut` returns the mutable value (if any) and a boolean representing whether the
-    /// value was found or not. The value can be nil and the boolean can be true at
-    /// the same time.
-    pub fn get_mut(&self, key: &K) -> Option<ValueRefMut<V, SS>> {
+    /// `get_mut` returns a `Option<ValueRefMut<V, SS>>` (if any) representing whether the
+    /// value was found or not.
+    pub fn get_mut(&self, key: &K) -> Option<ValueRefMut<V, S>> {
         if self.is_closed.load(Ordering::SeqCst) {
             return None;
         }
@@ -749,8 +835,8 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
         }
     }
 
-    // GetTTL returns the TTL for the specified key if the
-    // item was found and is not expired.
+    /// Returns the TTL for the specified key if the
+    /// item was found and is not expired.
     pub fn get_ttl(&self, key: &K) -> Option<Duration> {
         let (index, conflict) = self.key_to_hash.build_key(key);
         self.store
@@ -758,6 +844,7 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
             .and_then(|_| self.store.expiration(&index).map(|time| time.get_ttl()))
     }
 
+    /// clear the Cache.
     #[inline]
     pub fn clear(&self) -> Result<(), CacheError> {
         if self.is_closed.load(Ordering::SeqCst) {
@@ -818,6 +905,7 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
             self.insert_in(key, val, cost, Duration::ZERO, true).await
         }
 
+        /// wait until the previous operations finished.
         pub async fn wait(&self) -> Result<(), CacheError> {
             if self.is_closed.load(Ordering::SeqCst) {
                 return Ok(());
@@ -833,6 +921,7 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
             }
         }
 
+        /// remove entry from Cache by key.
         pub async fn remove(&self, k: &K) {
             if self.is_closed.load(Ordering::SeqCst) {
                 return;
@@ -876,7 +965,7 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
             if let Some((index, item)) = self.update(key, val, cost, ttl, only_update) {
                 let is_update = item.is_update();
                 // Attempt to send item to policy.
-                tokio::select! {
+                select! {
                     res = self.insert_buf_tx.send(item) => res.map_or_else(|_| {
                        if is_update {
                             // Return true if this was an update operation since we've already
@@ -933,6 +1022,7 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
             self.insert_in(key, val, cost, Duration::ZERO, true)
         }
 
+        /// wait until all the previous operations finished.
         pub fn wait(&self) -> Result<(), CacheError> {
             if self.is_closed.load(Ordering::SeqCst) {
                 return Ok(());
@@ -946,6 +1036,7 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
                 .map_err(|e| CacheError::SendError(format!("cache set buf sender: {}", e.to_string())))
         }
 
+        /// remove an entry from Cache by key.
         pub fn remove(&self, k: &K) {
             if self.is_closed.load(Ordering::SeqCst) {
                 return;
@@ -1058,27 +1149,23 @@ impl<K, V, KH, C, U, CB, PS, ES, SS> Cache<K, V, KH, C, U, CB, PS, ES, SS>
     }
 }
 
-impl<V, C, U, CB, PS, ES, SS> CacheProcessor<V, C, U, CB, PS, ES, SS>
+impl<V, U, CB, S> CacheProcessor<V, U, CB, S>
     where
         V: Send + Sync + 'static,
-        C: Coster<V>,
         U: UpdateValidator<V>,
         CB: CacheCallback<V>,
-        PS: BuildHasher + Clone + 'static,
-        ES: BuildHasher + Clone + 'static,
-        SS: BuildHasher + Clone + 'static,
+        S: BuildHasher + Clone + 'static,
 {
     fn new(
         num_to_keep: usize,
         ignore_internal_cost: bool,
         cleanup_duration: Duration,
-        store: Arc<ShardedMap<V, U, SS, ES>>,
-        policy: Arc<LFUPolicy<PS>>,
+        store: Arc<ShardedMap<V, U, S, S>>,
+        policy: Arc<LFUPolicy<S>>,
         insert_buf_rx: Receiver<Item<V>>,
         stop_rx: Receiver<()>,
         clear_rx: UnboundedReceiver<()>,
         metrics: Arc<Metrics>,
-        coster: Arc<C>,
         callback: Arc<CB>,
     ) -> Self {
         let item_size = store.item_size();
@@ -1093,7 +1180,6 @@ impl<V, C, U, CB, PS, ES, SS> CacheProcessor<V, C, U, CB, PS, ES, SS>
             num_to_keep,
             callback,
             ignore_internal_cost,
-            coster,
             item_size,
             cleanup_duration
         }
@@ -1305,18 +1391,15 @@ impl<V, C, U, CB, PS, ES, SS> CacheProcessor<V, C, U, CB, PS, ES, SS>
 
 }
 
-impl<'a, V, C, U, CB, PS, ES, SS> CacheCleaner<'a, V, C, U, CB, PS, ES, SS>
+impl<'a, V, U, CB, S> CacheCleaner<'a, V, U, CB, S>
     where
         V: Send + Sync + 'static,
-        C: Coster<V>,
         U: UpdateValidator<V>,
         CB: CacheCallback<V>,
-        PS: BuildHasher + Clone + 'static,
-        ES: BuildHasher + Clone + 'static,
-        SS: BuildHasher + Clone + 'static,
+        S: BuildHasher + Clone + 'static,
 {
     #[inline]
-    fn new(processor: &'a mut CacheProcessor<V, C, U, CB, PS, ES, SS>) -> Self {
+    fn new(processor: &'a mut CacheProcessor<V, U, CB, S>) -> Self {
         Self { processor }
     }
 
@@ -1343,7 +1426,7 @@ impl<'a, V, C, U, CB, PS, ES, SS> CacheCleaner<'a, V, C, U, CB, PS, ES, SS>
 
     cfg_async! {
         #[inline]
-        pub(crate) async fn clean(mut self) -> Result<(), CacheError> {
+        async fn clean(mut self) -> Result<(), CacheError> {
             loop {
                 select! {
                     // clear out the insert buffer channel.
